module Wicoris
  module Postman
    class FaxMachine
      DIALOUT_PREFIX = '0'
      VALID_PHONE_NUMBER = %r{
        ^0       # 1st char has to be a zero
        [1-9]{1} # 2nd char has to be a digit execpt zero
        \d{6,}$  # followed by at least 6 other digits
      }x

      attr_reader :logger

      def initialize(job, opts = {})
        @job = job
        @opts = opts
        @logger = opts[:logger]
      end

      # Actually fax the letter.
      def run
        Mixlib::ShellOut.new(command).run_command unless @opts[:noop]
        logger.debug :message => 'Letter faxed',
                     :phone => validated_phone,
                     :letter => @job.letter,
                     :job => @job
      end

      # @returns [String] Validated phone number.
      def validated_phone
        raise ArgumentError, 'Missing phone number' unless @job.phone
        phone = @job.phone.gsub(/(\s|-)+/, '')
        if phone =~ VALID_PHONE_NUMBER
          DIALOUT_PREFIX + phone
        else
          raise ArgumentError, "Invalid phone number: #{phone}"
        end
      end

      private

      # Return the command-line for sending the fax, i.e.:
      #
      #   lp -d Fax -o phone=042 "/tmp/foo.pdf"
      #
      # @returns [String] command-line
      def command
        cmd = %w(lp -d Fax) # TODO: Replace hard-coded fax printer-name 'Fax'!
        cmd << "-o phone=#{validated_phone}"
        cmd << "'#{@job.letter}'"
        cmd.join(' ')
      end
    end
  end
end
