module Wicoris
  module Postman
    class Postman
      def initialize(opts = {})
        @opts = opts
        @logger = opts[:logger]
      end

      # Process each job.
      def run
        jobs.each do |job|
          begin
            job.process
          rescue => e
            @logger.error(e) if @logger
          ensure
            job.clear!
          end
        end
      end

      private

      # @returns [Array<Job>] All jobs created each JSON file
      def jobs
        json_files.map { |f| Job.new(f, @opts) }
      end

      # @returns [Array<String>] JSON files in jobdir.
      def json_files
        # NOTE: This performs case-insensitive globbing.
        Dir.glob(File.join(jobdir, 'AB_*.JSON'), File::FNM_CASEFOLD)
      end

      # @returns [String] Path to jobdir directory
      def jobdir
        @opts[:jobdir]
      end
    end
  end
end
